@@ -36,6 +36,7 @@ let init_flags () =
   begin
 (*** The following flags control search. ***)
     Hashtbl.add bool_flags "ENABLE_PATTERN_CLAUSES" false;
+    Hashtbl.add bool_flags "PATTERN_CLAUSES_EARLY" false; (*** Dec 2012 - If true, make a pattern clause for the formula when it's assigned a literal instead of when it's processed. ***)
     Hashtbl.add bool_flags "DYNAMIC_PATTERN_CLAUSES" true; (*** Create pattern clauses from universally quantified formulas during search. - Chad, April 6, 2011 ***)
     Hashtbl.add bool_flags "PATTERN_CLAUSES_TRANSITIVITY_EQ" false; (*** Make pattern clauses for transitivity of equality. ***)
     Hashtbl.add bool_flags "PATTERN_CLAUSES_FORALL_AS_LIT" true;
@@ -119,6 +120,7 @@ let init_flags () =
     Hashtbl.add int_flags "AXIOM_DELAY" 0; (*** Set this to > 0 to work on the negated conjecture first ***)
     Hashtbl.add int_flags "RELEVANCE_DELAY" 0; (*** Set this to > 0 to delay axioms that do not share names with the negated conjecture. Only has an effect if TREAT_CONJECTURE_SPECIAL is true ***)
     Hashtbl.add bool_flags "ALL_DEFS_AS_EQNS" false;
+
     Hashtbl.add bool_flags "EAGERLY_PROCESS_INSTANTIATIONS" true;
     Hashtbl.add int_flags "INSTANTIATION_DELAY" 1; (*** If EAGERLY_PROCESS_INSTANTIATIONS is false ***)
     Hashtbl.add int_flags "ARTP_WEIGHT" 1; (*** If EAGERLY_PROCESS_INSTANTIATIONS is false ***)
@@ -139,6 +141,7 @@ let init_flags () =
     Hashtbl.add int_flags "EQ_TP_WEIGHT_FAC" 1; (*** If EAGERLY_PROCESS_INSTANTIATIONS is false ***)
     Hashtbl.add int_flags "CHOICE_WEIGHT" 1; (*** If EAGERLY_PROCESS_INSTANTIATIONS is false ***)
     Hashtbl.add int_flags "CHOICE_TP_WEIGHT_FAC" 1; (*** If EAGERLY_PROCESS_INSTANTIATIONS is false ***)
+
     Hashtbl.add int_flags "NOT_IN_PROP_MODEL_DELAY" 0; (*** Nov 2011: Use propositional approximation to focus work on those true in the current propositional model. Koen Claessen suggested this at CADE 23. ***)
     Hashtbl.add bool_flags "HOUNIF1" false; (*** Mar 2012 - use HO unification to suggest instantiations during search ***)
     Hashtbl.add bool_flags "HOUNIF1MATE" false;
@@ -158,4 +161,20 @@ let init_flags () =
     Hashtbl.add bool_flags "PFTRM_ADD_SYM_CLAUSES" true;
     Hashtbl.add bool_flags "PFTRM_PRESORT_CLAUSES" true;
     Hashtbl.add bool_flags "PFTRM_REMOVE_INDEPENDENT" true;
+
+(*** The flag MINISAT_SEARCH_PERIOD controls how often MiniSat asked to search for unsatisfiability.
+     Default of 1 means every time a new clause is given to MiniSat. ***)
+    Hashtbl.add int_flags "MINISAT_SEARCH_PERIOD" 1;
+
+(*** Send first-order formulas to E with timeout of E_TIMEOUT seconds every E_PERIOD times a first-order formula is encountered. ***)
+    Hashtbl.add bool_flags "USE_E" false;
+    Hashtbl.add int_flags "E_PERIOD" 100;
+    Hashtbl.add int_flags "E_TIMEOUT" 1;
+    Hashtbl.add bool_flags "E_EQO_EQUIV" false;
+
+(*** Feb 18 2013
+ - INSTANTIATION_ORDER_CYCLE - how long to cycle a stack/queue switching pattern.
+ - INSTANTIATION_ORDER_MASK - integer describing the stack/queue switching pattern. ***)
+    Hashtbl.add int_flags "INSTANTIATION_ORDER_CYCLE" 1;
+    Hashtbl.add int_flags "INSTANTIATION_ORDER_MASK" 0;
   end
