@@ -108,11 +108,20 @@ let process_thf f =
       else
 	declare_thf_logic_formula name role (formula_to_pretrm f)
   | Appl(Symbol "$$include",((Symbol file)::_)) -> (*** ignoring select ***)
-      begin
-	if (!verbosity > 1) then Printf.printf "include file %s\n" file;
-	!st_include_fun (String.sub file 1 (String.length file - 2)); (*** Remove the quotes ***)
-	if (!verbosity > 1) then Printf.printf "included file %s\n" file;
-      end
+      let file2 = String.sub file 1 (String.length file - 2) in (*** Remove the quotes ***)
+      let dir2 = Filename.dirname file2 in
+      if (dir2 = ".") then
+	begin
+	  if (!verbosity > 1) then Printf.printf "include file %s\n" file;
+	  !st_include_fun (String.sub file 1 (String.length file - 2)); (*** Remove the quotes ***)
+	  if (!verbosity > 1) then Printf.printf "included file %s\n" file;
+	end
+      else
+	begin
+	  if (!verbosity > 1) then Printf.printf "include file %s\n" file;
+	  !st_find_read_thf_fun dir2 file2;
+	  if (!verbosity > 1) then Printf.printf "included file %s\n" file;
+	end
   | _ -> raise (GenericSyntaxError("Unhandled TPTP declaration. " ^ (Formula.to_string f)))
 
 let accumulator = ref (fun f f_list -> f::f_list)
