@@ -244,10 +244,17 @@ let print_coq_proofscript c r =
     end
   else
     begin
-      if (result_coq && size < maxcoqproofsize ) then ref_coq c prenorm_refutation else raise (CoqProofTooBig size)
+      if !result_coq then
+        if size < maxcoqproofsize then
+          ref_coq c prenorm_refutation
+        else raise (CoqProofTooBig size)
+      else if !result_isabellehol then
+        if size < max_isabellehol_proofsize then
+          ref_isabellehol c prenorm_refutation
+        else raise (CoqProofTooBig size) (*FIXME replace with IsabelleHOLProofTooBig*)
     end;
   let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
-  if (result_coq ) then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
+  if !result_coq then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
 	(*** Output Latex Search ***)
 
   if (result_latex && width < 50 && depth < 30) then
@@ -321,9 +328,9 @@ let print_coq_sproofterm c r =
 	(*** Output Coq ***)
 
   let beforeCoq= Sys.time() in
-  if (result_coq && size < maxcoqproofsize ) then ref_coq_spfterm c prenorm_refutation else raise (CoqProofTooBig size);
+  if (!result_coq && size < maxcoqproofsize ) then ref_coq_spfterm c prenorm_refutation else raise (CoqProofTooBig size);
   let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
-  if (result_coq ) then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
+  if !result_coq then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
 	(*** Output Latex Search ***)
 
   if (result_latex && width < 50 && depth < 30) then
@@ -504,7 +511,29 @@ let print_hocore c r =
     end
 
 (*** Chad - July 2012 - output proof script in tstp format. ***)
-let print_tstp c r =
+let print_tstp c (r : State.refut) =
   tstp := true;
-  print_coq_proofscript c r
 
+  (*To conform to the TSTP syntax spec, we first need to print the
+    signature: types and constants. Extract this from the refutation.*)
+  let consts = consts_of_refut [] r in
+  let types = base_types_of_refut [] r in
+
+  if (!verbosity > vold) then
+    begin
+      print_endline ("|consts| = " ^ string_of_int (List.length consts));
+      print_endline ("|types| = " ^ string_of_int (List.length types))
+    end;
+
+  List.map (fun basetype_name ->
+    output_string c
+      ("thf(ty" ^ basetype_name ^ ", type, " ^ basetype_name ^ " : $tType).\n"))
+   types;
+
+  List.map (fun (name, stp) ->
+    let tstp_name = tstpizename name in
+    output_string c ("thf(ty" ^ tstp_name ^ ", type, " ^ tstp_name ^ " : ");
+    Coq.print_stp_tstp stdout stp true;
+    output_string c (").\n"))
+   consts;
+  print_coq_proofscript c r
