@@ -16,7 +16,7 @@ exception Initialbranch_Hypothesis_Error
 let beforeSearch = ref 0.0
 let isTimeout = ref false 
 
-let myConflict n (a,b) = if debug_search then Printf.printf "myConflict %d %s %s\n" n (trm_str a) (trm_str b); Conflict(a,b) (** delete me **)
+let myConflict n (a,b) = if debug_search then Printf.printf "myConflict %d %s %s\n" n (trm_str a) (trm_str b); Conflict(a,b)
 
 (** HEURISTIK  **)  
 
