@@ -842,17 +842,25 @@ let rec leib_rewrite_refutation m p porig n r c =
 	if debug_translation then Printf.printf "is_an_eqn_p returned with \nq %s\n" (trm_str q);
 	Rewrite(q,e1,e2,leib_rewrite_refutation_2 (onlybetanorm (Ap(q,e2))) porig 0 r)
       with Not_found ->
-	match (m,p) with
-	| (Ap(m1,m2),Ap(p1,p2)) ->
-	    leib_rewrite_refutation m1 p1 porig n (leib_rewrite_refutation m2 p2 porig n r (fun z -> c (Ap(p1,z)))) (fun z -> c (Ap(z,m2)))
-	| (Lam(a,m1),Lam(b,p1)) when a = b ->
-	    leib_rewrite_refutation m1 p1 porig (n + 1) r (fun z -> c (Lam(a,z)))
-	| (Lam(a,m1),Ap(Eq(b),DB(0,_))) when a = b ->
-	    leib_rewrite_refutation m1 (Ap(Ap(Eq(a),DB(1,a)),DB(0,a))) porig (n + 1) r (fun z -> c (Lam(a,z)))
-	| (Lam(a,m1),Eq(b)) when a = b ->
-	    leib_rewrite_refutation m1 (Ap(Eq(b),DB(0,a))) porig (n + 1) r (fun z -> c (Lam(a,z)))
-	| _ ->
-	    raise (GenericError ("Cannot show " ^ (trm_str m) ^ " and " ^ (trm_str p) ^ " are equal in leib_rewriting refutation."))
+(***  July 2012: Need to add case for exists rewritten to choice. Currently modes with EXISTSTOCHOICE true will often not produce proofs.
+	try
+	  let (pred,e1,e2,aoa) = exists_to_choice_p m p n in
+	  let q = Lam(aoa,c pred) in
+	  if debug_translation then Printf.printf "exists_to_choice_p returned with \nq %s\n" (trm_str q);
+	  Rewrite(q,e1,e2,leib_rewrite_refutation_2 (onlybetanorm (Ap(q,e2))) porig 0 r)
+	with Not_found ->
+***)
+	  match (m,p) with
+	  | (Ap(m1,m2),Ap(p1,p2)) ->
+	      leib_rewrite_refutation m1 p1 porig n (leib_rewrite_refutation m2 p2 porig n r (fun z -> c (Ap(p1,z)))) (fun z -> c (Ap(z,m2)))
+	  | (Lam(a,m1),Lam(b,p1)) when a = b ->
+	      leib_rewrite_refutation m1 p1 porig (n + 1) r (fun z -> c (Lam(a,z)))
+	  | (Lam(a,m1),Ap(Eq(b),DB(0,_))) when a = b ->
+	      leib_rewrite_refutation m1 (Ap(Ap(Eq(a),DB(1,a)),DB(0,a))) porig (n + 1) r (fun z -> c (Lam(a,z)))
+	  | (Lam(a,m1),Eq(b)) when a = b ->
+	      leib_rewrite_refutation m1 (Ap(Eq(b),DB(0,a))) porig (n + 1) r (fun z -> c (Lam(a,z)))
+	  | _ ->
+	      raise (GenericError ("Cannot show " ^ (trm_str m) ^ " and " ^ (trm_str p) ^ " are equal in leib_rewriting refutation."))
     end
 and leib_rewrite_refutation_2 m p n r =
   if (m = p) then
