@@ -185,7 +185,7 @@ let print_help () =
    print_newline();
    print_string "-G : A Coq file containing multiple conjectures is given. Try to prove each of them independently.";
    print_newline();
-   print_string "-p [tstp|coqscript|coqspfterm|hocore|modeltrue|model]: Output a kind of proof object";
+   print_string "-p [tstp|coqscript|coqspfterm|hocore|modeltrue|model|isar]: Output a kind of proof object";
    print_newline());;
 
 (*** Load the mode (error if cannot find it) ***)
@@ -221,10 +221,8 @@ and read_mode_file_value mf flagname =
       read_mode_file mf
     end
 
-let modedir = ref (Config.satallaxdir ^ "/modes")
-  
 let load_mode m =
-  let modefile = (!modedir ^ "/" ^ m) in
+  let modefile = (!Config.modedir ^ "/" ^ m) in
   if (Sys.file_exists modefile) then
     begin
       try
@@ -339,7 +337,7 @@ let process_command_line_args () =
 	    incr i;
 	    let mdir = Sys.argv.(!i) in
 	    slaveargs := (mdir::"-M"::!slaveargs);
-	    modedir := mdir
+	    Config.modedir := mdir
 	  end
       | "-t" -> (incr i; timeout := (Some (float_of_string Sys.argv.(!i))))
       | "-slave" -> (incr i; slave := true)
@@ -376,6 +374,12 @@ let process_command_line_args () =
 	      mkproofterm := Some Model
 	    else if ((String.lowercase pfkind) = "modeltrue") then
 	      mkproofterm := Some ModelTrue
+	    else if ((String.lowercase pfkind) = "isar") then
+        begin
+	        mkproofterm := Some IsarScript;
+          Flag.result_coq := false;
+          Flag.result_isabellehol := true
+        end
 	    else
 	      raise (InputHelpError ("Unknown kind of proof " ^ pfkind ^ " for -p"))
 	  end
@@ -457,6 +461,14 @@ let search_main () =
 		      coqoutchannel := stdout;
 		    end;
 		end
+	    | Some IsarScript -> (*FIXME code in this section, and in related sections, need refactoring. names are a bit misleading.*)
+		begin
+		  if (not (!coq)) then
+		    begin
+		      coq := true;
+		      coqoutchannel := stdout;
+		    end;
+		end
 	    | _ -> ()
 	  end;
 	  if (!verbosity > 8) then print_flags () else ();
@@ -553,11 +565,32 @@ let search_main () =
 		    print_string "% Higher-Order Unsat Core BEGIN"; print_newline();
 		    print_hocore stdout r;
 		    print_string "% Higher-Order Unsat Core END"; print_newline();
+		| (Some IsarScript, Some r) ->
+		    if (!coqoutchannel = stdout) then
+		      begin
+			      print_string "(*% SZS output start Proof*)"; print_newline();
+			      print_string "(*% Isar Proof Script*)"; print_newline()
+		      end;
+		    let c = !coqoutchannel in
+		      begin
+		        try print_coq_proofscript c r
+		        with CoqProofTooBig coqproofsize -> (*FIXME naming is misleading*)
+			        begin
+			          if (!verbosity > 0) then Printf.printf "%% SZS status Success\nIsar Proof Too Big: %d steps\n" coqproofsize;
+			          exit 26;
+			        end
+		      end;
+		      if (c = stdout) then
+		        begin
+			        print_string "(*% SZS output end Proof*)"; print_newline();
+		        end
+		      else
+		        close_out c;
 		| _ -> ()
 	      end;
 	      match !conjecture with
-	      | Some _ -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Theorem\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 20)
-	      | None -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Unsatisfiable\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 25)
+	      | Some _ -> (if (!verbosity > 0) then (Printf.printf "(*%% SZS status Theorem*)\n"; Printf.printf "(*%% %s *)\n" (String.concat " " (!mode))); exit 20)
+	      | None -> (if (!verbosity > 0) then (Printf.printf "(*%% SZS status Unsatisfiable*)\n"; Printf.printf "(*%% %s *)\n" (String.concat " " (!mode))); exit 25)
 	    end
 	    else
 	      raise Timeout (*** if nontheorem was set, then assume we timedout here. ***)
