@@ -8,6 +8,25 @@ open Syntax
 open Match
 open Minisatinterface
 
+let minisatsearchcounter = ref 0;;
+let minisatsearchperiod = ref 1;;
+
+let enable_pattern_clauses = ref false;;
+let dynamic_pattern_clauses = ref false;;
+
+let minisat_search_period () =
+  incr minisatsearchcounter;
+  if (!minisatsearchcounter >= !minisatsearchperiod) then
+    begin
+      minisatsearchcounter := 0;
+      if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
+      let r = minisat_search () in
+      if ((!verbosity > 3) && r) then begin Printf.printf "still satisfiable\n"; flush stdout end;
+      r
+    end
+  else
+    true
+
 (*** Compute minimal unsatisfiable core with picomus - This should be moved somewhere more reasonable. - Chad, Nov 9, 2010 ***)
 let minimal_unsatisfiable_core clauses =
   if (!verbosity > 4) then (Printf.printf "About to call picomus %s\n" (!Config.picomus); flush stdout);
@@ -106,14 +125,11 @@ let rec new_assumption_clause c =
       |	[] -> raise (Unsatisfiable None)
       |	[l1] ->
 	  begin
-	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause1 (%d)\n" l1; flush stdout end;
+	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause1 a (%d)\n" l1; flush stdout end;
 	    if (minisat_addClause1(l1)) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable a\n"; flush stdout end)
-		else
-		  raise (Unsatisfiable None)
+		if (not (minisat_search_period())) then
+		  raise (Unsatisfiable None);
 	      end
 	    else raise (Unsatisfiable None)
 	  end
@@ -124,10 +140,7 @@ let rec new_assumption_clause c =
 	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause (%s)\n" (List.fold_left (fun x y -> if ((String.length x) == 0) then (string_of_int y) else (x ^ " " ^ (string_of_int y))) "" c); flush stdout end;
 	    if (minisat_addClause()) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable b\n"; flush stdout end)
-		else
+		if (not (minisat_search_period())) then
 		  raise (Unsatisfiable None)
 	      end
 	    else raise (Unsatisfiable None)
@@ -174,13 +187,10 @@ let rec new_search_clause c r =
       |	[] -> raise (Unsatisfiable None)
       |	[l1] ->
 	  begin
-	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause1 (%d)\n" l1; flush stdout end;
+	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause1 b (%d)\n" l1; flush stdout end;
 	    if (minisat_addClause1(l1)) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable c\n"; flush stdout end)
-		else
+		if (not (minisat_search_period())) then
 		  raise (Unsatisfiable None)
 	      end
 	    else raise (Unsatisfiable None)
@@ -190,10 +200,7 @@ let rec new_search_clause c r =
 	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause2 (%d %d)\n" l1 l2; flush stdout end;
 	    if (minisat_addClause2 l1 l2) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable d\n"; flush stdout end)
-		else
+		if (not (minisat_search_period())) then
 		  raise (Unsatisfiable None)
 	      end
 	    else raise (Unsatisfiable None)
@@ -203,10 +210,7 @@ let rec new_search_clause c r =
 	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause3 (%d %d %d)\n" l1 l2 l3; flush stdout end;
 	    if (minisat_addClause3 l1 l2 l3) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable e\n"; flush stdout end)
-		else
+		if (not (minisat_search_period())) then
 		  raise (Unsatisfiable None)
 	      end
 	    else raise (Unsatisfiable None)
@@ -219,10 +223,7 @@ let rec new_search_clause c r =
 	    if (!verbosity > 3) then begin Printf.printf "About to call minisat_addClause (%s)\n" (List.fold_left (fun x y -> if ((String.length x) == 0) then (string_of_int y) else (x ^ " " ^ (string_of_int y))) "" c); flush stdout end;
 	    if (minisat_addClause()) then
 	      begin
-		if (!verbosity > 3) then begin Printf.printf "About to call minisat_search\n"; flush stdout end;
-		if (minisat_search()) then
-		  (if (!verbosity > 3) then begin Printf.printf "still satisfiable f\n"; flush stdout end)
-		else
+		if (not (minisat_search_period())) then
 		  raise (Unsatisfiable None)
 	      end
 	    else raise (Unsatisfiable None)
@@ -935,7 +936,22 @@ and new_pattern_clauses_2a nalllit m mr evars strict unmatched abslits remp remi
 (*** nalllit is something the literal of the negation of a forall formula; m is the forall formula ***)
 let new_pattern_clauses nalllit m =
  let mm = to_meta m in
- new_pattern_clauses_2 nalllit [mm] [] [] [] [] [] [] (fun _ _ _ _ -> ())
+ new_pattern_clauses_2 nalllit [mm] [] [] [] [] [] [] (fun _ _ _ _ -> ());;
+
+let get_literal_fn1 : (trm -> int) ref = ref get_literal;;
+let get_literal_fn2 : (trm -> int) ref = ref get_literal;;
+
+(*** Dec 2012: Make pattern clauses when the formula is assigned a literal. ***)
+let get_literal_and_make_pattern_clause m =
+  match m with
+  | (Ap(Forall(_),_)) ->
+      if assignedp m then
+	get_literal m
+      else
+	let l = get_literal m in
+	new_pattern_clauses l m;
+	l
+  | _ -> get_literal m
 
 let sym_eq_clauses_1 mlit msymlit a m1 m2 =
   let symab = Lam(a,forall a (imp (eq a (DB(1,a)) (DB(0,a))) (eq a (DB(0,a)) (DB(1,a))))) in
@@ -1059,7 +1075,7 @@ let process_instantiation a m =
 	if (not (filteredp (-lit))) then
 	  let pm = (norm name_def (ap(p,m))) in
 	  insertWithPriority (get_int_flag "FORALL_DELAY") (ProcessProp1 pm);
-	  let pmlit = get_literal pm in
+	  let pmlit = !get_literal_fn2 pm in
 	  new_search_clause [lit;pmlit]
 	    (if (mkprooftermp ()) then (Some(InstRule(a,p,m))) else None)
 	    )
@@ -1087,6 +1103,10 @@ let possibly_new_instantiation a m =
       see_inst a m;
       new_active_instantiation a m
     end
+  else if (!verbosity > 5) then
+    begin
+      Printf.printf "not really new inst %s\n" (trm_str m)
+    end
 
 let process_new_name a m d =
   let (sigmal,ar) = argtps_rtp a in
@@ -1101,8 +1121,8 @@ let negimp_rule mlit m m1 m2 =
   let r = if (mkprooftermp ()) then (Some(NegPropRule(m))) else None in
   insertWithPriority (get_int_flag "POST_NOR_L_DELAY") (ProcessProp1 m1);
   insertWithPriority (get_int_flag "POST_NOR_R_DELAY") (ProcessProp1 (normneg m2));
-  let m1lit = get_literal m1 in
-  let nm2lit = get_literal (normneg m2) in
+  let m1lit = !get_literal_fn2 m1 in
+  let nm2lit = !get_literal_fn2 (normneg m2) in
   new_search_clause [-mlit;m1lit] r;
   new_search_clause [-mlit;nm2lit] r
 
@@ -1110,15 +1130,15 @@ let imp_rule mlit m m1 m2 =
   let r = if (mkprooftermp ()) then (Some(PosPropRule(m))) else None in
   insertWithPriority (get_int_flag "POST_OR_L_DELAY") (ProcessProp1 (normneg m1));
   insertWithPriority (get_int_flag "POST_OR_R_DELAY") (ProcessProp1 m2);
-  let nm1lit = get_literal (normneg m1) in
-  let m2lit = get_literal m2 in
+  let nm1lit = !get_literal_fn2 (normneg m1) in
+  let m2lit = !get_literal_fn2 m2 in
   new_search_clause [(-mlit);nm1lit;m2lit] r
 
 let forall_rule nmlit a m1 w =
   if (!verbosity > 5) then begin print_string("Instantiating with " ^ (trm_str w)); print_newline(); flush stdout end;
   let m1w = (norm name_def (ap(m1,w))) in
   insertWithPriority (get_int_flag "FORALL_DELAY") (ProcessProp1 m1w);
-  let m1wlit = get_literal m1w in
+  let m1wlit = !get_literal_fn2 m1w in
   new_search_clause [nmlit;m1wlit]
     (if (mkprooftermp ()) then (Some(InstRule(a,m1,w))) else None)
 
@@ -1130,7 +1150,7 @@ let negforall_rule mlit a m1 =
   if (!verbosity > 8) then begin print_string(" m1w = " ^ (trm_str m1w)); print_newline(); flush stdout end;
   let nm1w = (normneg m1w) in
   insertWithPriority (get_int_flag "EXISTS_DELAY") (ProcessProp1 nm1w);
-  let nm1wlit = get_literal nm1w in
+  let nm1wlit = !get_literal_fn2 nm1w in
   new_search_clause [-mlit;nm1wlit]
     (if (mkprooftermp ()) then (Some(FreshRule(a,m1,ws))) else None)
 
@@ -1140,7 +1160,7 @@ let decompose lit largs rargs r =
       (fun la ra ->
 	let deq = normneg (ueq la ra) in
 	insertWithPriority delay (ProcessProp1 deq);
-	get_literal deq
+	!get_literal_fn2 deq
       )
       largs rargs
   in
@@ -1163,6 +1183,7 @@ let consider_confrontation a neqlit u v eqlit s t =
   insertWithPriority delay op
 
 let confront a eqlit s t neqlit u v =
+  if (!verbosity > 9) then Printf.printf "Confront %d %d\n" eqlit neqlit;
   let r = if (mkprooftermp ()) then (Some (ConfrontationRule(eqlit,neqlit))) else None in
   let minuseqlit = (- eqlit) in
   let minusneqlit = (- neqlit) in
@@ -1170,10 +1191,10 @@ let confront a eqlit s t neqlit u v =
   let tu = (eq a t u) in
   let sv = (eq a s v) in
   let tv = (eq a t v) in
-  let msulit = (- (get_literal su)) in
-  let mtulit = (- (get_literal tu)) in
-  let msvlit = (- (get_literal sv)) in
-  let mtvlit = (- (get_literal tv)) in
+  let msulit = (- (!get_literal_fn2 su)) in
+  let mtulit = (- (!get_literal_fn2 tu)) in
+  let msvlit = (- (!get_literal_fn2 sv)) in
+  let mtvlit = (- (!get_literal_fn2 tv)) in
   insertWithPriority (get_int_flag "POST_CONFRONT1_DELAY") (ProcessProp1 (neg su));
   insertWithPriority (get_int_flag "POST_CONFRONT2_DELAY") (ProcessProp1 (neg tu));
   insertWithPriority (get_int_flag "POST_CONFRONT3_DELAY") (ProcessProp1 (neg sv));
@@ -1193,7 +1214,7 @@ let choice_rule h args =
 	    let m2 = norm name_def (forall a (normneg (ap(shift pred 1 0,DB(0,a))))) in
 	    insertWithPriority (get_int_flag "CHOICE_EMPTY_DELAY") (ProcessProp1 m2);
 	    insertWithPriority (get_int_flag "CHOICE_IN_DELAY") (ProcessProp1 m1);
-	    new_search_clause [get_literal m1;get_literal m2] (if (mkprooftermp ()) then (Some(ChoiceRule(h,pred))) else None)
+	    new_search_clause [!get_literal_fn2 m1;!get_literal_fn2 m2] (if (mkprooftermp ()) then (Some(ChoiceRule(h,pred))) else None)
 	| _ -> raise (GenericError "Choice operator must be applied to at least one argument")
       end
   | None -> ()  
@@ -1335,7 +1356,7 @@ let generate_types d =
   if (!verbosity > 9) then (Printf.printf "Generate types called with depth %d\n" d; flush stdout);
   if (d = (get_int_flag "ENUM_O")) then generate_type d Prop;
   if (d = (get_int_flag "ENUM_SORT")) then
-    (Hashtbl.iter (fun a _ -> generate_type d (Base(a))) name_base);
+    (List.iter (fun a -> generate_type d (Base(a))) !name_base_list);
   let dm = d - 1 - (get_int_flag "ENUM_ARROW") in
   if (dm >= 0) then
     begin
@@ -1587,8 +1608,8 @@ let process_search_option op =
       begin
 	if (not (Hashtbl.mem processed m)) then
 	  begin
-	    let mlit = get_literal m in
-	    if ((!not_in_prop_model_delay_p) && ((minisat_modelValue mlit) > 0)) then (*** Chad: Nov 2011 - delay if not true in the current prop approx ***)
+	    let mlit = !get_literal_fn2 m in
+	    if ((!not_in_prop_model_delay_p) && (!minisatsearchcounter = 0) && ((minisat_modelValue mlit) > 0)) then (*** Chad: Nov 2011 - delay if not true in the current prop approx; Chad: Dec 2012 - added condition that minisatsearchcounter = 0 so that modelValue is only called if the last interaction with minisat actually called search. Otherwise we may get segmentation faults. The effect is that if MINISAT_SEARCH_PERIOD > 1 and NOT_IN_PROP_MODEL_DELAY > 0, then we only check if the prop is in the model when we happen to have called minisat to search. ***)
 	      begin
 		if (!verbosity > 8) then Printf.printf "using prop model to delay working on %d %s\n" mlit (trm_str m);
 		insertWithPriority (!not_in_prop_model_delay) op
@@ -1600,13 +1621,13 @@ let process_search_option op =
 		match m with
 		| Ap(Ap(Imp,Ap(Forall(a),m1)),False) ->
 		    begin
-		      if ((get_bool_flag "ENABLE_PATTERN_CLAUSES") && (get_bool_flag "PATTERN_CLAUSES_FORALL_AS_LIT")) then (apply_pattern_clauses mlit m);
+		      if ((!enable_pattern_clauses) && (get_bool_flag "PATTERN_CLAUSES_FORALL_AS_LIT")) then (apply_pattern_clauses mlit m);
 		      negforall_rule mlit a m1
 		    end
 		| Ap(Ap(Imp,nm),False) ->
 		    let (h,args) = head_spine nm in
 		    begin
-		      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+		      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 		      match h with
 		      | False -> () (*** do nothing ***)
 		      | Imp ->
@@ -1635,11 +1656,11 @@ let process_search_option op =
 					  possibly_new_instantiation a m1;
 					  possibly_new_instantiation a m2
 					end;
-				      let m3lit = get_literal m3 in
+				      let m3lit = !get_literal_fn2 m3 in
 				      new_search_clause [-mlit;m3lit] r
 				  | Prop -> (*** BE ***)
-				      let m1lit = get_literal m1 in
-				      let m2lit = get_literal m2 in
+				      let m1lit = !get_literal_fn2 m1 in
+				      let m2lit = !get_literal_fn2 m2 in
 				      insertWithPriority (get_int_flag "POST_NEQO_L_DELAY") (ProcessProp1 m1);
 				      insertWithPriority (get_int_flag "POST_NEQO_R_DELAY") (ProcessProp1 m2);
 				      insertWithPriority (get_int_flag "POST_NEQO_NL_DELAY") (ProcessProp1 (normneg m1));
@@ -1725,7 +1746,7 @@ let process_search_option op =
 		    end
 		| Ap(Forall(a),m1) -> (*** Instantiate ***)
 		    begin
-		      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m; if (get_bool_flag "DYNAMIC_PATTERN_CLAUSES") then new_pattern_clauses (-mlit) m);
+		      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m; if (!dynamic_pattern_clauses) then new_pattern_clauses (-mlit) m);
 		      if ((get_bool_flag "FILTER_UNIV") && (filterp mlit)) then
 			()
 		      else
@@ -1758,8 +1779,8 @@ let process_search_option op =
 					      let d_def = get_int_flag "IMITATE_DEFN_DELAY" in
 					      begin
 						enum_started := true;
-						Hashtbl.iter
-						  (fun x ((m,sigma),_) ->
+						List.iter
+						  (fun (x,m,sigma) ->
 						    let (sigmal,rtp) = argtps_rtp sigma in
 						    if (Hashtbl.mem name_def x) then
 						      begin
@@ -1769,7 +1790,7 @@ let process_search_option op =
 						    else
 						      new_usable_head_rtp rtp sigmal m d_const
 							)
-						  name_trm
+						  !name_trm_list
 					      end;
 					    enum_of_start a;
 					    insertWithPriority d (EnumIterDeep(get_int_flag "ENUM_ITER_DEEP_INIT",a))
@@ -1788,11 +1809,11 @@ let process_search_option op =
 					    begin
 					      enum_started := true;
 					      insertWithPriority (d + d_o) (EnumTp(d_o,Prop,Prop));
-					      Hashtbl.iter
-						(fun beta _ -> insertWithPriority (d + d_i) (EnumTp(d_i,Base(beta),Base(beta))))
-						name_base;
-					      Hashtbl.iter
-						(fun x ((m,sigma),_) ->
+					      List.iter
+						(fun beta -> insertWithPriority (d + d_i) (EnumTp(d_i,Base(beta),Base(beta))))
+						!name_base_list;
+					      List.iter
+						(fun (x,m,sigma) ->
 						  let (sigmal,rtp) = argtps_rtp sigma in
 						  if (Hashtbl.mem name_def x) then
 						    begin
@@ -1802,7 +1823,7 @@ let process_search_option op =
 						  else
 						    new_usable_head_rtp rtp sigmal m d_const
 						      )
-						name_trm
+						!name_trm_list
 					    end
 					end;
 					begin
@@ -1836,11 +1857,11 @@ let process_search_option op =
 		      match h with
 		      | False ->
 			  let r = if (mkprooftermp ()) then (Some(PosPropRule(m))) else None in
-			  let flit = get_literal False in
+			  let flit = !get_literal_fn2 False in
 			  new_search_clause [-flit] r
 		      | Imp ->
 			  begin
-			    if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+			    if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 			    match args with
 			      [m1;m2] -> imp_rule mlit m m1 m2
 			    | _ -> raise (GenericError "Imp should have had exactly two arguments; there's a bug")
@@ -1857,11 +1878,11 @@ let process_search_option op =
 				  if ((get_bool_flag "PATTERN_CLAUSES_TRANSITIVITY_EQ") && (not (Hashtbl.mem pattern_clauses_transitivity_types a))) then
 				    begin (*** April 6, 2011 - add a pattern clause for transitivity of equality the first time we see an equation at a type. ***)
 				      let transa = forall a (forall a (forall a (imp (eq a (DB(2,a)) (DB(1,a))) (imp (eq a (DB(1,a)) (DB(0,a))) (eq a (DB(2,a)) (DB(0,a))))))) in
-				      let transalit = get_literal transa in
+				      let transalit = !get_literal_fn2 transa in
 				      let transa2 = forall a (forall a (forall a (imp (eq a (DB(2,a)) (DB(1,a))) (imp (eq a (DB(0,a)) (DB(1,a))) (eq a (DB(2,a)) (DB(0,a))))))) in
-				      let transa2lit = get_literal transa2 in
+				      let transa2lit = !get_literal_fn2 transa2 in
 				      let transa3 = forall a (forall a (forall a (imp (eq a (DB(1,a)) (DB(2,a))) (imp (eq a (DB(1,a)) (DB(0,a))) (eq a (DB(2,a)) (DB(0,a))))))) in
-				      let transa3lit = get_literal transa3 in
+				      let transa3lit = !get_literal_fn2 transa3 in
 				      Hashtbl.add pattern_clauses_transitivity_types a ();
 				      pattern_clauses_forall_as_lit := false;
 				      pattern_clauses_onlyallstrict := false;
@@ -1876,16 +1897,16 @@ let process_search_option op =
 				    end;
 				  match a with
 				  | Ar(a1,a2) -> (*** FQ ***)
-				      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+				      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 (***				  let m3 = forall a1 (eq a2 (norm name_def (Ap (shift m1 0 1,DB (0,a1)))) (norm name_def (Ap (shift m2 0 1,DB (0,a1))))) in ***)
 				      let m3 = norm name_def (forall a1 (eq a2 (Ap (m1,DB (0,a1))) (Ap (m2,DB (0,a1))))) in (*** See FE above for why there's no shift on m1 and m2. Mar 15 2011: Normalize after everything, to make sure the case of a final eta is handled, e.g., forall x.s=x where s has no x free eta reduces. ***)
 				      insertWithPriority (get_int_flag "POST_NFEQ_DELAY") (ProcessProp1 m3);
-				      let m3lit = get_literal m3 in
+				      let m3lit = !get_literal_fn2 m3 in
 				      new_search_clause [-mlit;m3lit] r
 				  | Prop -> (*** BQ ***)
-				      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
-				      let m1lit = get_literal m1 in
-				      let m2lit = get_literal m2 in
+				      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
+				      let m1lit = !get_literal_fn2 m1 in
+				      let m2lit = !get_literal_fn2 m2 in
 				      insertWithPriority (get_int_flag "POST_EQO_L_DELAY") (ProcessProp1 m1);
 				      insertWithPriority (get_int_flag "POST_EQO_R_DELAY") (ProcessProp1 m2);
 				      insertWithPriority (get_int_flag "POST_EQO_NL_DELAY") (ProcessProp1 (normneg m1));
@@ -1897,7 +1918,7 @@ let process_search_option op =
 					()
 				      else
 					begin
-					  if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+					  if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 					  Hashtbl.add peqns aname (mlit,m1,m2);
 					  List.iter
 					    (fun (otherlit,n1,n2) -> (*** (delayed) confrontation ***)
@@ -1918,7 +1939,7 @@ let process_search_option op =
 			    ()
 			  else
 			    begin
-			      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+			      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 			      Hashtbl.add pchoiceatoms a (mlit,args);
 			      List.iter (fun (nlit,nargs) ->
 				if (not (args = nargs)) then
@@ -1937,7 +1958,7 @@ let process_search_option op =
 			    ()
 			  else
 			    begin
-			      if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then (apply_pattern_clauses mlit m);
+			      if (!enable_pattern_clauses) then (apply_pattern_clauses mlit m);
 			      if (decomposable x) then
 				begin
 				  let pmd = get_int_flag "PRE_MATING_DELAY_POS" in
@@ -1962,12 +1983,13 @@ let process_search_option op =
 	  end (*** no else ***)
       end
   | Mating(plit,nlit,pargs,nargs) ->
+      if (!verbosity > 9) then Printf.printf "Mating %d %d\n" plit nlit;
       let pmd = get_int_flag "POST_MATING_DELAY" in
       let dlits = List.map2
 	  (fun pa na ->
 	    let deq = normneg (ueq pa na) in
 	    insertWithPriority pmd (ProcessProp1 deq);
-	    get_literal deq
+	    !get_literal_fn2 deq
 	  )
 	  pargs nargs
       in
@@ -2131,6 +2153,12 @@ let search_main () =
 	  print_model false
       | _ -> ()
     end;
+    if (!minisatsearchperiod > 1) then
+      begin
+	let r = minisat_search () in
+	if (not r) then
+	  raise (Unsatisfiable None)
+      end;
     raise Satisfiable
 
 let pos_heads : (trm,unit) Hashtbl.t = Hashtbl.create 10
@@ -2237,7 +2265,7 @@ let search_1 b1 b2 =
 	    insertWithPriority 0 (ProcessProp1 mn);
 	    if ((get_int_flag "RELEVANCE_DELAY") > 0) then
 	      set_relevance_info mn false;
-	    let mlit = get_literal mn in
+	    let mlit = !get_literal_fn1 mn in
 	    new_assumption_clause [mlit];
 	    if (get_bool_flag "INITIAL_SUBTERMS_AS_INSTANTIATIONS") then
 	      begin
@@ -2260,7 +2288,7 @@ let search_1 b1 b2 =
 	    insertWithPriority
 		((get_int_flag "AXIOM_DELAY") + (relevance_delay mn true (get_int_flag "RELEVANCE_DELAY")))
 	      (ProcessProp1 mn);
-	    let mlit = get_literal mn in
+	    let mlit = !get_literal_fn1 mn in
 	    new_assumption_clause [mlit];
 	    if (get_bool_flag "INITIAL_SUBTERMS_AS_INSTANTIATIONS") then
 	      begin
@@ -2281,7 +2309,7 @@ let search_1 b1 b2 =
       begin
         Hashtbl.iter (fun _ axs ->
 			  (List.iter (fun ax ->
-					  let axlit = get_literal ax in
+					  let axlit = !get_literal_fn1 ax in
 					  new_assumption_clause [axlit];
 					  insertWithPriority (get_int_flag "AXIOM_DELAY") (ProcessProp1 ax);
 					  if (!verbosity > 1) then Printf.printf "Putting axiom on queue %s\n" (trm_str ax)
@@ -2523,6 +2551,26 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
       end
 
 let search () =
+  minisatsearchperiod := get_int_flag "MINISAT_SEARCH_PERIOD";
+  useeprover := get_bool_flag "USE_E";
+  eeqoequiv := get_bool_flag "E_EQO_EQUIV";
+  eproverperiod := get_int_flag "E_PERIOD";
+  eprovertimeout := get_int_flag "E_TIMEOUT";
+  if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then
+    begin
+      enable_pattern_clauses := true;
+      if (get_bool_flag "DYNAMIC_PATTERN_CLAUSES") then
+	dynamic_pattern_clauses := true;
+      if (get_bool_flag "PATTERN_CLAUSES_EARLY") then
+	begin
+	  get_literal_fn1 := get_literal_and_make_pattern_clause;
+	  if (!dynamic_pattern_clauses) then
+	    begin
+	      get_literal_fn2 := get_literal_and_make_pattern_clause;
+	      dynamic_pattern_clauses := false
+	    end
+	end
+    end;
   begin
     if (not (!nontheorem)) then
       begin
