@@ -47,7 +47,7 @@ let rec updatecoqglobalsectionstack cx cgss co =
 
 let conjecturename : string ref = ref "claim"
 let conjecture : (pretrm * trm * trm) option ref = ref None
-type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
+type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue | IsarScript
 let mkproofterm = ref None
 let mkprooftermp () = match !mkproofterm with Some _ -> true | None -> false
 let slave1 = ref false
@@ -66,9 +66,37 @@ let name_trm_list : (string * trm * stp) list ref = ref []
 let name_def : (string,trm) Hashtbl.t = Hashtbl.create 100
 let name_def_prenorm : (string,trm) Hashtbl.t = Hashtbl.create 100
 let name_hyp : (string,trm) Hashtbl.t = Hashtbl.create 100
-let coq_used_names : (string,unit) Hashtbl.t = Hashtbl.create 100
-let coq_names : (string,string) Hashtbl.t = Hashtbl.create 100
-let coq_hyp_names : (string,string) Hashtbl.t = Hashtbl.create 100
+
+(** Replaces all occurences of 'Neg' by 'implies False' **)
+let rec negnorm1 m =
+  match m with
+  | Ap(Neg,m1) ->
+      let (n1,_) = negnorm1 m1 in
+      (imp n1 False,true)
+  | Neg -> (Lam(Prop,imp (DB(0,Prop)) False),true)
+  | Ap(m1,m2) ->
+      let (n1,b1) = negnorm1 m1 in
+      let (n2,b2) = negnorm1 m2 in
+      if (b1 || b2) then
+	(Ap(n1,n2),true)
+      else
+	(m,false)
+  | Lam(a1,m1) ->
+      let (n1,b1) = negnorm1 m1 in
+      if b1 then
+	(Lam(a1,n1),true)
+      else
+	(m,false)
+  | _ -> (m,false)
+(** applies neg- and betanormalization**)
+let onlynegnorm m =
+  let (n,_) = negnorm1 m in onlybetanorm n
+(** applies neg-, beta- and delta-normalization**)
+let coqnorm m =
+  let m = betanorm name_def_prenorm m in
+  let (n,_) = negnorm1 m in n
+(** applies full satallax normalization**)
+let normalize pt = norm name_def (logicnorm pt)
 
 let coqknown (x,y) =
   if (!coq2) then y else x
@@ -912,106 +940,174 @@ let coq_init () =
     Hashtbl.add coq_used_names "Sepsilon_spec" ();
     (*** Other names to avoid ***)
     Hashtbl.add coq_used_names "claim" ();
+
+    (*FIXME add isar keywords*)
+    Hashtbl.add coq_used_names "thm" ();
+    Hashtbl.add coq_used_names "lemma" ();
   end
 
 let print_coqsig c =
   let rec print_coqsig_base l =
     match l with
-      (x::r) ->
-	print_coqsig_base r;
-	if (not (!coq2)) then Printf.fprintf c "Variable %s:SType.\n" x
-    | [] -> ()
+        (x :: r) ->
+	        print_coqsig_base r;
+          if !mkproofterm = Some IsarScript then
+            if x <> "i" (*FIXME "$i"="i"*) then
+              (*there's no need to redeclare $i*)
+              (*FIXME to avoid name clashes, could suffix names with something*)
+              Printf.fprintf c "typedecl %s\n" x
+            else ()
+          else
+	          if (not (!coq2)) then Printf.fprintf c "Variable %s:SType.\n" x
+      | [] -> ()
   in
   let rec print_coqsig_const l =
     match l with
-      ((x,a)::r) ->
-	begin
-	  print_coqsig_const r;
-	  try
-	    ignore (List.assoc x (!coqsig_def));
-	  with
-	  | Not_found ->
-	      begin
-		try
-		  Printf.fprintf c "Variable %s : " x;
-		  if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
-		  Printf.fprintf c ".\n"
-		with
-		| Not_found ->
-		    begin
-		      if (c != stdout) then close_out c;
-		      raise (GenericError("A Satallax bug caused a problem creating the Coq file."))
-		    end
-	      end
-	end
-    | [] -> ()
+        ((x, a) :: r) ->
+	        begin
+	          print_coqsig_const r;
+	          try
+	            ignore (List.assoc x (!coqsig_def));
+	          with
+	            | Not_found ->
+	                begin
+		                try
+                      if !mkproofterm = Some IsarScript then
+                        begin
+		                      Printf.fprintf c "consts %s :: \"" x;
+		                      print_stp_isar c a (* coq_names *) false;
+		                      Printf.fprintf c "\"\n"
+                        end
+                      else
+                        begin
+		                      Printf.fprintf c "Variable %s : " x;
+		                      if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
+		                      Printf.fprintf c ".\n"
+                        end
+		                with
+		                  | Not_found ->
+		                      begin
+		                        if (c != stdout) then close_out c;
+		                        raise (GenericError("A Satallax bug caused a problem creating the Coq/Isar file."))
+		                      end
+	                end
+	        end
+      | [] -> ()
   in
   let rec print_coqsig_def l =
     match l with
-      ((x,a)::r) ->
-	begin
-	  print_coqsig_def r;
-	  try
-	    let m = List.assoc x (!coqsig_def) in
-	    begin
-	      Printf.fprintf c "Definition %s : " x;
-	      if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
-	      Printf.fprintf c "\n := ";
-	      if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
-	      Printf.fprintf c ".\n"
-	    end
-	  with
-	  | Not_found -> ()
-	end
-    | [] -> ()
+        ((x,a)::r) ->
+	        begin
+	          print_coqsig_def r;
+	          try
+	            let m = List.assoc x (!coqsig_def) in
+                if !mkproofterm = Some IsarScript then
+	                begin
+	                  Printf.fprintf c "definition %s :: \"" x;
+	                  print_stp_isar c a (* coq_names *) false;
+	                  Printf.fprintf c "\"\n where \"%s == (" x;
+	                  print_pretrm_isar c m coq_names coq_used_names (-1) (-1);
+	                  Printf.fprintf c ")\"\n"
+	                end
+                else
+	                begin
+	                  Printf.fprintf c "Definition %s : " x;
+	                  if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
+	                  Printf.fprintf c "\n := ";
+	                  if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
+	                  Printf.fprintf c ".\n"
+	                end
+	          with
+	            | Not_found -> ()
+	        end
+      | [] -> ()
   in
   let rec print_coqsig_hyp l =
     match l with
-      ((x,m)::r) ->
-	begin
-	  try
-	    print_coqsig_hyp r;
-	    Printf.fprintf c "Hypothesis %s : " x;
-	    if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
-	    Printf.fprintf c ".\n"
-	  with
-	  | Not_found ->
-	      begin
-		if (c != stdout) then close_out c;
-		raise (GenericError("A Satallax bug caused a problem creating the Coq file."))
-	      end
-	end
-    | [] -> ()
+        ((x, t)::r) ->
+	        begin
+	          try
+              if !mkproofterm = Some IsarScript then
+	              begin
+	                print_coqsig_hyp r;
+	                Printf.fprintf c "assumes %s : \"" x;
+	                (* print_pretrm_isar c m coq_names coq_used_names (-1) (-1); *)
+                  trm_to_isar c (coqnorm t) (Syntax.Variables.make ());
+	                Printf.fprintf c "\"\n"
+                end
+              else if !mkproofterm = Some CoqScript || !mkproofterm = Some TSTP then
+                (*have Syntax.trm but need Syntax.pretrm, so look it up*)
+                let pt = List.assoc x !coqsig_hyp in
+	                begin
+	                  print_coqsig_hyp r;
+	                  Printf.fprintf c "Hypothesis %s : " x;
+	                  if (!coq2) then print_pretrm_coq2 c pt (-1) (-1) else print_pretrm_coq c pt coq_names coq_used_names (-1) (-1);
+	                  Printf.fprintf c ".\n"
+                  end
+              else
+                failwith "Printing of hypotheses: Unrecognised proof-output format."
+	          with
+	            | Not_found ->
+	                begin
+		                if (c != stdout) then close_out c;
+		                raise (GenericError("A Satallax bug caused a problem creating the Coq file."))
+	                end
+	        end
+      | [] -> ()
   in
-  if (not (!coqlocalfile)) then
-  begin
-    begin
-      match (!mkproofterm) with
-      |	Some CoqSPfTerm ->
-	  begin
-	    Printf.fprintf c "Require Export stt3.\n";
-	  end
-      |	Some CoqScript ->
-	  begin
-	    Printf.fprintf c "Add LoadPath \"%s/coq\".\n" Config.satallaxdir;
-	    Printf.fprintf c "Require Import stttab.\n";
-	    Printf.fprintf c "Section SatallaxProblem.\n"
-	  end
-      |	_ -> ()
-    end;
-    print_coqsig_base !coqsig_base;
-    print_coqsig_const !coqsig_const;
-    print_coqsig_def !coqsig_const;
-    print_coqsig_hyp !coqsig_hyp;
-    match (!conjecture) with
-    | Some (m,_,_) ->
-	begin
-	  Printf.fprintf c "Theorem %s : " (Hashtbl.find coq_hyp_names (!conjecturename));
-	  if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
-	  Printf.fprintf c ".\n"
-	end
-    | None -> Printf.fprintf c "Theorem claim : False.\n"
-  end
+    if (not (!coqlocalfile)) then
+      begin
+        begin
+          match (!mkproofterm) with
+            |	Some CoqSPfTerm ->
+	              begin
+	                Printf.fprintf c "Require Export stt3.\n";
+	              end
+            |	Some CoqScript ->
+	              begin
+	                Printf.fprintf c "Add LoadPath \"%s/coq\".\n" Config.satallaxdir;
+	                Printf.fprintf c "Require Import stttab.\n";
+	                Printf.fprintf c "Section SatallaxProblem.\n"
+	              end
+            |	Some IsarScript ->
+	              begin
+	                Printf.fprintf c "theory SatallaxProblem\n";
+	                Printf.fprintf c "imports Satallax\n";
+	                Printf.fprintf c "begin\n"
+	              end
+            |	_ -> ()
+        end;
+        print_coqsig_base !coqsig_base;
+        print_coqsig_const !coqsig_const;
+        print_coqsig_def !coqsig_const;
+        if !mkproofterm = Some IsarScript then
+	        Printf.fprintf c "\nlemma\n";
+        print_coqsig_hyp !coqsig_hyp_trm;
+        match (!conjecture) with
+          | Some (m,t,_) ->
+              if !mkproofterm = Some IsarScript then
+	              begin
+	                Printf.fprintf c "shows %s : \"" (Hashtbl.find coq_hyp_names (!conjecturename));
+	                (* print_pretrm_isar c m coq_names coq_used_names (-1) (-1); *)
+	                trm_to_isar c (coqnorm t) (Syntax.Variables.make ());
+	                Printf.fprintf c "\"\n";
+
+                  (*FIXME currently all definitions are unfolded, irrespective of whether when they're used. This seems to reflect Satallax's usage anyway.*)
+                  if List.length !coqsig_def > 0 then
+	                  Printf.fprintf c "unfolding %s\n" (String.concat " " (List.map (fun (s, _) -> s ^ "_def") !coqsig_def))
+	              end
+              else
+	              begin
+	                Printf.fprintf c "Theorem %s : " (Hashtbl.find coq_hyp_names (!conjecturename));
+	                if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
+	                Printf.fprintf c ".\n"
+	              end
+          | None ->
+              if !mkproofterm = Some IsarScript then
+                Printf.fprintf c "shows claim : \"False\"\n"
+              else
+                Printf.fprintf c "Theorem claim : False.\n"
+      end
 
 let declare_base_type a =
   if (!coq) then
@@ -1331,4 +1427,121 @@ let print_num_axioms () =
   let n = List.length (!initial_branch) in
   Printf.printf "(NUMAXIOMS \"%s\" %d)\n" (!problemfile) n;
   exit 123
-  
+
+
+(** Extraction of constants and types from different kinds of types. **)
+
+(*List the constants that occur in a rule-info, as name-type pairs.
+  NOTE the resulting list doesn't contain duplicate constants.*)
+let rec consts_of_ruleinfo acc : ruleinfo -> ctx = function
+    NegPropRule t
+  | PosPropRule t
+  | FreshRule (_, t, _) -> consts_of_trm acc t
+  | InstRule (_, t1, t2)
+  | ChoiceRule (t1, t2) ->
+    let acc' = consts_of_trm acc t1
+    in consts_of_trm acc' t2
+  | ConfrontationRule (_, _)
+  | MatingRule (_, _)
+  | Known (_, _, _) -> acc
+
+(*List the constants that occur in a refutation, as name-type pairs.
+  NOTE the resulting list doesn't contain duplicate constants.*)
+let rec consts_of_refut acc : refut -> ctx = function
+    NegImpR (t1, t2, r)
+  | EqFR (_, _, t1, t2, r)
+  | NegEqFR (_, _, t1, t2, r) ->
+    let acc' = consts_of_trm acc t1 in
+    let acc'' = consts_of_trm acc' t2 in
+    consts_of_refut acc'' r
+  | NegAllR (stp, t, name, r) ->
+    let acc' =
+      begin
+        print_endline name;
+      try
+        let stp' = List.assoc name acc
+        in
+          if stp' <> stp then failwith "Different types for same constant?"
+          else acc
+      with Not_found ->
+        (name, stp) :: acc
+      end in
+    let acc'' = consts_of_trm acc' t in
+    consts_of_refut acc'' r
+  | EqOR (t1, t2, r1, r2)
+  | NegEqOR (t1, t2, r1, r2)
+  | ImpR (t1, t2, r1, r2) ->
+    let acc' = consts_of_trm acc t1 in
+    let acc'' = consts_of_trm acc' t2 in
+    let acc''' = consts_of_refut acc'' r1 in
+    consts_of_refut acc''' r2
+  | FalseR
+  | NegReflR
+  | AssumptionConflictR -> acc
+  | SearchR (cl, cr) ->
+    List.fold_right (fun c acc ->
+      let acc' =
+        List.fold_right (fun z acc ->
+          consts_of_trm acc (Hashtbl.find atom_hash_rev (abs z)))
+          c acc in
+      try
+        consts_of_ruleinfo acc' (cr c)
+      with Not_found -> acc') cl acc
+
+(*List the base types that are referenced in a rule-info.
+  NOTE the resulting list doesn't contain duplicate elements.*)
+let rec base_types_of_ruleinfo acc : ruleinfo -> string list = function
+    NegPropRule t
+  | PosPropRule t -> base_types_of_trm acc t
+  | FreshRule (stp, t, _) ->
+    let acc' = base_types acc stp
+    in base_types_of_trm acc' t
+  | InstRule (stp, t1, t2) ->
+    let acc' = base_types_of_trm acc t1 in
+    let acc'' = base_types_of_trm acc' t2
+    in base_types acc'' stp
+  | Known (_, _, stps) ->
+    List.fold_right (fun x y -> base_types y x) stps acc
+  | ChoiceRule (t1, t2) ->
+    let acc' = base_types_of_trm acc t1
+    in base_types_of_trm acc' t2
+  | ConfrontationRule (_, _)
+  | MatingRule (_, _) -> acc
+
+(*List the base types that are referenced in a refutation.
+  NOTE the resulting list doesn't contain duplicate elements.*)
+let rec base_types_of_refut acc : refut -> string list = function
+    NegImpR (t1, t2, r) ->
+    let acc' = base_types_of_trm acc t1 in
+    let acc'' = base_types_of_trm acc' t2 in
+    base_types_of_refut acc'' r
+  | EqFR (stp1, stp2, t1, t2, r)
+  | NegEqFR (stp1, stp2, t1, t2, r) ->
+    let acc' = base_types_of_trm acc t1 in
+    let acc'' = base_types_of_trm acc' t2 in
+    let acc3 = base_types acc'' stp1 in
+    let acc4 = base_types acc3 stp2 in
+    base_types_of_refut acc4 r
+  | NegAllR (stp, t, _, r) ->
+    let acc' = base_types acc stp in
+    let acc'' = base_types_of_trm acc' t in
+    base_types_of_refut acc'' r
+  | EqOR (t1, t2, r1, r2)
+  | NegEqOR (t1, t2, r1, r2)
+  | ImpR (t1, t2, r1, r2) ->
+    let acc' = base_types_of_trm acc t1 in
+    let acc'' = base_types_of_trm acc' t2 in
+    let acc''' = base_types_of_refut acc'' r1 in
+    base_types_of_refut acc''' r2
+  | FalseR
+  | NegReflR
+  | AssumptionConflictR -> acc
+  | SearchR (cl, cr) ->
+    List.fold_right (fun c acc ->
+      let acc' =
+        List.fold_right (fun z acc ->
+          base_types_of_trm acc (Hashtbl.find atom_hash_rev (abs z)))
+          c acc in
+      try
+        base_types_of_ruleinfo acc' (cr c)
+      with Not_found -> acc') cl acc
