@@ -41,7 +41,7 @@ val updatecoqglobalsectionstack : prectx -> (string * (out_channel -> unit) * pr
 
 val conjecturename : string ref
 val conjecture : (pretrm * trm * trm) option ref
-type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
+type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue | IsarScript
 val mkproofterm : proofkind option ref
 val mkprooftermp : unit -> bool
 val slave1 : bool ref
@@ -60,9 +60,6 @@ val name_trm_list : (string * trm * stp) list ref
 val name_def : (string,trm) Hashtbl.t
 val name_def_prenorm : (string,trm) Hashtbl.t
 val name_hyp : (string,trm) Hashtbl.t
-val coq_used_names : (string,unit) Hashtbl.t
-val coq_names : (string,string) Hashtbl.t
-val coq_hyp_names : (string,string) Hashtbl.t
 val coqknown : string * string -> string
 
 exception Timeout
@@ -75,6 +72,7 @@ val require : string -> unit
 val get_fresh_name : stp -> string * trm
 
 val max_atom : unit -> int
+val atom_hash_rev : (int, trm) Hashtbl.t
 
 val assignedp : trm -> bool
 
@@ -254,4 +252,11 @@ val reset_search : unit -> unit
 
 val print_model : bool -> unit
 
+val onlynegnorm : trm -> trm
+val coqnorm : trm -> trm
+val normalize : trm -> trm
 
+val base_types_of_ruleinfo : string list -> ruleinfo -> string list
+val base_types_of_refut : string list -> refut -> string list
+val consts_of_ruleinfo : ctx -> ruleinfo -> ctx
+val consts_of_refut : ctx -> refut -> ctx
