@@ -256,7 +256,7 @@ let coqify_name x h1 h2 =
   Hashtbl.add h2 y ();
   y
   
-let rec print_stp_coq c m h p =
+let rec print_stp_coq c m h p = (*FIXME this is also defined in coq.ml*)
   match m with
   | Base x ->
       Printf.fprintf c "%s" (Hashtbl.find h x)
@@ -271,6 +271,42 @@ let rec print_stp_coq c m h p =
 	if p then Printf.fprintf c ")"
       end
 
+(*
+let rec print_stp_isar c m h p =
+  match m with
+    | Base x ->
+	      let x = try (Hashtbl.find h x) with Not_found -> failwith("print_stp_isar can't find coqname of "^x) in
+          Printf.fprintf c "%s" x
+    | Prop ->
+        Printf.fprintf c "o"
+    | Ar(a,b) ->
+        begin
+	        if p then Printf.fprintf c "(";
+	        print_stp_isar c a h true;
+	        Printf.fprintf c " => ";
+	        print_stp_isar c b h false;
+	        if p then Printf.fprintf c ")"
+        end
+*)
+let rec print_stp_isar c m p =
+  match m with
+    | Base x ->
+        if x <> "$i" then
+          Printf.fprintf c "%s" x
+        else
+          Printf.fprintf c "i"(*FIXME this may clash with a base type that's really called "i"*)
+    | Prop ->
+        Printf.fprintf c "o"
+    | Ar(a,b) ->
+        begin
+(*	        if p then*) Printf.fprintf c "(";
+	        print_stp_isar c a true;
+	        Printf.fprintf c "=>";
+	        print_stp_isar c b false;
+(*	        if p then*) Printf.fprintf c ")";
+	        flush c
+        end
+
 let rec print_pretrm_coq c m h hu lp rp =
   match m with
   | PName x ->
@@ -615,6 +651,371 @@ and print_ex_coq c xl m h hu =
 	print_pretrm_coq c m h hu (-1) (-1)
       end
 
+(*FIXME repeating the code is a bad idea. this is repeated from print_pretrm_coq*)
+let rec print_pretrm_isar c m h hu lp rp =
+  match m with
+    | PName x ->
+        Printf.fprintf c "%s" (Hashtbl.find h x)
+    | PType -> (* Printf.fprintf c "SType" *) ()
+    | PPropTp -> Printf.fprintf c "o"
+    | PIndTp ->
+        Printf.fprintf c "%s" (Hashtbl.find h "$i") (*FIXME what's set as $i? where is this set?*)  
+    | PTrue -> Printf.fprintf c "True"
+    | PFalse -> Printf.fprintf c "False"
+    | PAp(PNeg,m1) ->
+        if ((lp < 0) && (rp < 30)) then
+	        begin
+	          Printf.fprintf c "~ ";
+	          print_pretrm_isar c m1 h hu 30 rp;
+	        end
+        else
+	        begin
+	          Printf.fprintf c "(~ ";
+	          print_pretrm_isar c m1 h hu 30 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PForall,m1) ->
+        if ((lp < 0) && (rp < 0)) then
+	        begin
+	          (* Printf.fprintf c "SPi _ "; *)
+	          Printf.fprintf c "All";
+	          print_pretrm_isar c m1 h hu (-1) (-1);
+	        end
+        else
+	        begin
+	          (* Printf.fprintf c "(SPi _ "; *)
+	          Printf.fprintf c "(All ";
+	          print_pretrm_isar c m1 h hu (-1) (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PExists,m1) ->
+        if ((lp < 0) && (rp < 0)) then
+	        begin
+	          (* Printf.fprintf c "SSigma _ "; *)
+	          Printf.fprintf c "Ex ";
+	          print_pretrm_isar c m1 h hu (-1) (-1);
+	        end
+        else
+	        begin
+	          (* Printf.fprintf c "(SSigma _ "; *)
+	          Printf.fprintf c "(Ex ";
+	          print_pretrm_isar c m1 h hu (-1) (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PImplies,m1),m2) ->
+(*
+        if ((lp < 17) && (rp < 16)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 17;
+	          Printf.fprintf c " --> ";
+	          print_pretrm_isar c m2 h hu 16 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 17;
+	          Printf.fprintf c " --> ";
+	          print_pretrm_isar c m2 h hu 16 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PRevImplies,m2),m1) ->
+(*
+        if ((lp < 17) && (rp < 16)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 17;
+	          Printf.fprintf c " --> ";
+	          print_pretrm_isar c m2 h hu 16 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 17;
+	          Printf.fprintf c " --> ";
+	          print_pretrm_isar c m2 h hu 16 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PAnd,m1),m2) ->
+(*
+        if ((lp < 21) && (rp < 20)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 21;
+	          Printf.fprintf c " & ";
+	          print_pretrm_isar c m2 h hu 20 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 21;
+	          Printf.fprintf c " & ";
+	          print_pretrm_isar c m2 h hu 20 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(POr,m1),m2) ->
+(*FIXME excluding parentheses can lead to parse problems on the Isabelle side
+        if ((lp < 19) && (rp < 18)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 19;
+	          Printf.fprintf c " | ";
+	          print_pretrm_isar c m2 h hu 18 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 19;
+	          Printf.fprintf c " | ";
+	          print_pretrm_isar c m2 h hu 18 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PIff,m1),m2) ->
+(*
+        if ((lp < 14) && (rp < 14)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 14;
+	          Printf.fprintf c " = ";
+	          print_pretrm_isar c m2 h hu 14 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 14;
+	          Printf.fprintf c " = ";
+	          print_pretrm_isar c m2 h hu 14 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PEq,m1),m2) ->
+(*
+        if ((lp < 40) && (rp < 40)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 40;
+	          Printf.fprintf c " = ";
+	          print_pretrm_isar c m2 h hu 40 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 40;
+	          Printf.fprintf c " = ";
+	          print_pretrm_isar c m2 h hu 40 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAp(PAp(PNEq,m1),m2) ->
+(*        if (rp >= 40) then*) Printf.fprintf c "(";
+        Printf.fprintf c "~ (";
+        print_pretrm_isar c m1 h hu (-1) 40;
+        Printf.fprintf c " = ";
+        print_pretrm_isar c m2 h hu 40 (-1);
+        Printf.fprintf c ")";
+(*        if (rp >= 40) then*) Printf.fprintf c ")";
+    | PAp(PAp(PNIff,m1),m2) ->
+(*        if (rp >= 30) then*) Printf.fprintf c "(";
+        Printf.fprintf c "~ (";
+        print_pretrm_isar c m1 h hu (-1) 14;
+        Printf.fprintf c " = ";
+        print_pretrm_isar c m2 h hu 14 (-1);
+        Printf.fprintf c ")";
+(*        if (rp >= 30) then*) Printf.fprintf c ")";
+    | PAp(PAp(PNAnd,m1),m2) ->
+(*        if (rp >= 30) then*) Printf.fprintf c "(";
+        Printf.fprintf c "~ (";
+        print_pretrm_isar c m1 h hu (-1) 20;
+        Printf.fprintf c " & ";
+        print_pretrm_isar c m2 h hu 21 (-1);
+        Printf.fprintf c ")";
+(*        if (rp >= 30) then*) Printf.fprintf c ")";
+    | PAp(PAp(PNOr,m1),m2) ->
+(*        if (rp >= 30) then*) Printf.fprintf c "(";
+        Printf.fprintf c "~ (";
+        print_pretrm_isar c m1 h hu (-1) 18;
+        Printf.fprintf c " | ";
+        print_pretrm_isar c m2 h hu 19 (-1);
+        Printf.fprintf c ")";
+(*        if (rp >= 30) then*) Printf.fprintf c ")";
+    | PIff -> Printf.fprintf c "(op =)"
+    | POr -> Printf.fprintf c "(op |)"
+    | PAnd -> Printf.fprintf c "(op &)"
+    | PNeg -> Printf.fprintf c "Not"
+    | PForall ->
+        Printf.fprintf c "All";
+    | PExists ->
+        Printf.fprintf c "Ex";
+    | PEq ->
+        Printf.fprintf c "(op =)";
+    | PAp(m1,m2) ->
+(*
+        if ((lp < 5000) && (rp < 5001)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 5000;
+	          Printf.fprintf c " ";
+	          print_pretrm_isar c m2 h hu 5001 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 5000;
+	          Printf.fprintf c " ";
+	          print_pretrm_isar c m2 h hu 5001 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | PAr(m1,m2) ->
+(*
+        if ((lp < 71) && (rp < 70)) then
+	        begin
+	          print_pretrm_isar c m1 h hu lp 71;
+	          Printf.fprintf c " => ";
+	          print_pretrm_isar c m2 h hu 70 rp;
+	        end
+        else
+*)
+	        begin
+	          Printf.fprintf c "(";
+	          print_pretrm_isar c m1 h hu (-1) 71;
+	          Printf.fprintf c " => ";
+	          print_pretrm_isar c m2 h hu 70 (-1);
+	          Printf.fprintf c ")";
+	        end
+    | POf(n,p) -> print_pretrm_isar c n h hu lp rp
+(***  | PDef(n,p) -> print_pretrm_isar c n h hu lp rp ***)
+    | PULam(xl,m) ->
+        begin
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c "(";
+	        begin
+	          Printf.fprintf c "%% ";
+	          print_ulam_isar c xl m h hu
+	        end;
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c ")";
+        end
+    | PLam(xl,m) ->
+        begin
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c "(";
+	        begin
+	          Printf.fprintf c "%% ";
+	          print_lam_isar c xl m h hu
+	        end;
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c ")";
+        end
+    | PAll(xl,m) ->
+        begin
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c "(";
+	        begin
+	          Printf.fprintf c "!";
+	          print_all_isar c xl m h hu
+	        end;
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c ")";
+        end
+    | PEx(xl,m) ->
+        begin
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c "(";
+	        begin
+	          print_ex_isar c xl m h hu
+	        end;
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c ")";
+        end
+    | PChoice((x,xa),m) ->
+        begin
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c "(";
+	        begin
+	          let y = coqify_name x h hu in
+	            Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	            Hashtbl.remove hu y; (*** free y to print the domain ***)
+	            Printf.fprintf c "Eps (%% ";
+	            Printf.fprintf c " %s" y;
+	            Printf.fprintf c "::";
+	            print_pretrm_isar c xa h hu (-1) (-1);
+	            Hashtbl.add h x y; (*** put y back on ***)
+	            Hashtbl.add hu y (); (*** put y back on ***)
+	            Printf.fprintf c " . ";
+	            print_pretrm_isar c m h hu (-1) (-1);
+	            Hashtbl.remove h x; (*** pops y from the values of x ***)
+	            Hashtbl.remove hu y; (*** free y to be used later ***)
+	            Printf.fprintf c ")";
+	        end;
+(*	        if ((lp >= 0) || (rp >= 0)) then*) Printf.fprintf c ")";
+        end
+    | _ -> raise (GenericSyntaxError ("Unknown pretrm case print Coq version : " ^ (pretrm_str m))) (*FIXME this wasnt changed from the original function*)
+and print_ulam_isar c xl m h hu =
+  match xl with
+      (x::xr) ->
+        begin
+	        let y = coqify_name x h hu in
+	          Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	          Hashtbl.remove hu y; (*** free y to print the domain ***)
+	          Printf.fprintf c " ";
+	          Printf.fprintf c "%s" y;
+	          Hashtbl.add h x y; (*** put y back on ***)
+	          Hashtbl.add hu y (); (*** put y back on ***)
+	          print_ulam_isar c xr m h hu;
+	          Hashtbl.remove h x; (*** pops y from the values of x ***)
+	          Hashtbl.remove hu y; (*** free y to be used later ***)
+        end
+    | [] ->
+        begin
+	        Printf.fprintf c " . ";
+	        print_pretrm_isar c m h hu (-1) (-1)
+        end
+and print_lam_isar c xl m h hu =
+  match xl with
+      ((x,a)::xr) ->
+        begin
+	        let y = coqify_name x h hu in
+	          Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	          Hashtbl.remove hu y; (*** free y to print the domain ***)
+	          Printf.fprintf c " ("; Printf.fprintf c "%s" y; Printf.fprintf c "::"; print_pretrm_isar c a h hu (-1) (-1); Printf.fprintf c ")";
+	          Hashtbl.add h x y; (*** put y back on ***)
+	          Hashtbl.add hu y (); (*** put y back on ***)
+	          print_lam_isar c xr m h hu;
+	          Hashtbl.remove h x; (*** pops y from the values of x ***)
+	          Hashtbl.remove hu y; (*** free y to be used later ***)
+        end
+    | [] ->
+        begin
+	        Printf.fprintf c " . ";
+	        print_pretrm_isar c m h hu (-1) (-1)
+        end
+and print_all_isar c xl m h hu =
+  match xl with
+      ((x,a)::xr) ->
+        begin
+	        let y = coqify_name x h hu in
+	          Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	          Hashtbl.remove hu y; (*** free y to print the domain ***)
+	          Printf.fprintf c " ("; Printf.fprintf c "%s" y; Printf.fprintf c "::"; print_pretrm_isar c a h hu (-1) (-1); Printf.fprintf c ")";
+	          Hashtbl.add h x y; (*** put y back on ***)
+	          Hashtbl.add hu y (); (*** put y back on ***)
+	          print_all_isar c xr m h hu;
+	          Hashtbl.remove h x; (*** pops y from the values of x ***)
+	          Hashtbl.remove hu y; (*** free y to be used later ***)
+        end
+    | [] ->
+        begin
+	        Printf.fprintf c ". ";
+	        print_pretrm_isar c m h hu (-1) (-1)
+        end
+and print_ex_isar c xl m h hu =
+  match xl with
+      ((x,a)::xr) ->
+        begin
+	        let y = coqify_name x h hu in
+	          Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	          Hashtbl.remove hu y; (*** free y to print the domain ***)
+	          Printf.fprintf c "?"; Printf.fprintf c " %s" y; Printf.fprintf c "::"; print_pretrm_isar c a h hu (-1) (-1); Printf.fprintf c ". ";
+	          Hashtbl.add h x y; (*** put y back on ***)
+	          Hashtbl.add hu y (); (*** put y back on ***)
+	          print_ex_isar c xr m h hu;
+	          Hashtbl.remove h x; (*** pops y from the values of x ***)
+	          Hashtbl.remove hu y; (*** free y to be used later ***)
+        end
+    | [] ->
+        begin
+	        print_pretrm_isar c m h hu (-1) (-1)
+        end
+
 let rec print_stp_coq2 c m p =
   match m with
   | Base _ ->
@@ -1448,16 +1849,31 @@ let neg_body m =
     Ap(Ap(Imp,n),False) -> Some n
   | _ -> None
 
+let eq_body m =
+  match m with
+      Ap (Ap (Eq a, x), y) -> Some (a, x, y)
+    | _ -> None
+
 let neg_p m =
   match (neg_body m) with Some _ -> true | None -> false
 
-let head_spine m =
-  let rec head_spine_aux m args =
-    match m with
-      Ap(f,a) -> head_spine_aux f (a::args)
-    | _ -> (m,args)
+(*This is like "head_spine" below, but accepts
+  a bound to the number of "Ap"s it is to go
+  through before stopping.*)
+let bounded_head_spine n m =
+  let rec head_spine_aux n m args =
+    if n = 0 then (m, args)
+    else
+      match m with
+          Ap (f, a) -> head_spine_aux (n - 1) f (a :: args)
+        | _ -> (m, args)
   in
-  head_spine_aux m []
+    head_spine_aux n m []
+
+(*Unbounded form of "bounded_head_spine": it
+  fully unfolds a term into the head (a non-applicative term)
+  and the spine of terms it is applied to.*)
+let head_spine m = bounded_head_spine (-1) m
 
 let rec rtp a =
   match a with
@@ -1574,3 +1990,223 @@ let rec trm_to_foform_stp_rec p vl eqoequiv =
 (*** Takes a trm p and either returns a pair of a foform with optionally a simple type
      or raises NotFO ***)
 let trm_to_foform_stp p eqoequiv = trm_to_foform_stp_rec p [] eqoequiv
+
+let tstpizename x =
+  if (String.length x > 0) then
+    if (String.get x 0 = '_') then
+      "eigen" ^ x
+    else if (String.get x 0 = '@') then
+      String.sub x 1 ((String.length x) - 1)
+    else
+      if ((String.length x > 5) && (String.sub x 0 5 = "eigen")) then
+	"not" ^ x
+      else
+	x
+  else x
+
+let coq_used_names : (string,unit) Hashtbl.t = Hashtbl.create 100
+let coq_names : (string,string) Hashtbl.t = Hashtbl.create 100
+let coq_hyp_names : (string,string) Hashtbl.t = Hashtbl.create 100
+(** The module Variables is used to translate DeBrujin Indices into variables**)
+module Variables = struct
+	(** next variable counter and list of used variable names**)
+	type t = int * (string list)
+	let make () = (1,[])
+	(** Input: Variables (n,v)
+		Output: Variables (m,v') with m>n and a new variable name x in v'  **)
+	let rec push (n,v) =
+		let x = "X" ^ (string_of_int n) in
+		let n = n+1 in
+		if (Hashtbl.mem coq_used_names x)
+			then push (n,v)
+  			else (n,x::v)
+	let top (_,v) = List.hd v
+	let get i (_,v) = List.nth v i
+end
+
+let isarize_name x =
+  if String.length x > 0 then
+    if String.get x 0 = '_' then
+      "eigen" ^ x
+    else if String.get x 0 = '@' then
+      String.sub x 1 ((String.length x) - 1)
+    else
+      if (String.length x > 5 && String.sub x 0 5 = "eigen") then
+        "not" ^ x
+      else
+        (*Rename constants to avoid clashes with keywords*)
+        let x =
+          try Hashtbl.find coq_names x with
+              Not_found -> x
+        in x
+  else failwith "Name cannot have zero length"
+
+(*FIXME next batch of functions are adapted from the TSTP functions*)
+let rec trm_to_isar c m bound =
+  match m with
+      Name(x,_) -> (* Definitions *)
+        Printf.fprintf c "%s" (isarize_name x)
+    | False -> (* Bottom *)
+	      Printf.fprintf c "False"
+    | Ap(Ap(Imp,m1),False) ->  (* Negation *)
+	      begin
+	        Printf.fprintf c "(~(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c "))";
+	      end
+    | Ap(Ap(Imp,m1),m2) -> (* Implication *)
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " --> ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")";
+	      end
+    | Ap(Imp,m1) -> trm_to_isar c (Lam(Prop,Ap(Ap(Imp,shift m1 0 1),DB(0,Prop)))) bound
+    | Imp -> trm_to_isar c (Lam(Prop,Lam(Prop,Ap(Ap(Imp,DB(1,Prop)),DB(0,Prop))))) bound
+    | Ap(Forall(a),Lam(_,m1)) -> (* forall with Lam *)
+        begin
+	        print_all_isar c a m1 bound
+        end
+    | Forall(a) ->
+        begin
+	        Printf.fprintf c "All";
+        end
+    | Ap(Ap(Eq(a),m1),m2) -> (* Equality *)
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " = ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")"
+	      end
+    | Eq(a) ->
+        Printf.fprintf c "(op =)"
+    | Ap(Choice(a),Lam(_,m1)) ->
+        let bound = Variables.push bound in
+          Printf.fprintf c "(@ "; Printf.fprintf c "%s" (Variables.top bound);
+          Printf.fprintf c "::"; print_stp_isar c a (*Hashtbl.create 0(*FIXME*)*) false;
+          Printf.fprintf c ". ";
+          trm_to_isar c m1 bound; Printf.fprintf c ")"
+    | Choice(a) ->
+        Printf.fprintf c "Eps";
+    | True -> (* Top *)
+	      Printf.fprintf c "True"
+    | Ap(Ap(And,m1),m2) -> (* conjunction *)
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " & ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")";
+	      end
+    | And ->Printf.fprintf c "(op &)"
+    | Ap(Ap(Or,m1),m2) -> (* disjunction *)
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " | ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")";
+	      end
+    | Or -> Printf.fprintf c "(op |)"
+    | Ap(Ap(Iff,m1),m2) -> (* equivalenz *)
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " = ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")";
+	      end
+    | Iff -> Printf.fprintf c "(op =)"
+    | Neg -> Printf.fprintf c "Not"
+    | Ap(Exists(a),Lam(_,m1)) -> (* exist *)
+        begin
+	        print_ex_isar c a m1 bound
+        end
+    | Exists(a) ->
+        begin
+	        Printf.fprintf c "Ex";
+        end
+    | DB(i,a) -> (* Bound variable *)
+	      Printf.fprintf c "%s" (Variables.get i bound)
+    | Lam(a,m) ->
+        begin
+	        print_lam_isar c a m bound
+        end
+    | Ap(m1,m2) ->
+	      begin
+	        Printf.fprintf c "(";
+	        trm_to_isar c m1 bound;
+	        Printf.fprintf c " ";
+	        trm_to_isar c m2 bound;
+	        Printf.fprintf c ")";
+	      end
+    | _ -> raise (GenericSyntaxError ("Unknown case in trm_to_isar version : " ^ (trm_str m)))
+ (* Prints consecutive lambda-terms as a single fun in Isar. *)
+and print_lam_isar c a m bound =
+	let bound = Variables.push bound in
+	  Printf.fprintf c "(%% "; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c "::"; print_stp_isar c a (*Hashtbl.create 0(*FIXME*)*) false; Printf.fprintf c ". ";
+	  match m with
+		  | Lam(b,m') -> print_lam_isar c b m' bound; Printf.fprintf c ")"
+		  | _ -> trm_to_isar c m bound; Printf.fprintf c ")"
+(* Prints consecutive forall-terms together with the corresponding lambda-terms as a single forall in Isar. *)
+and print_all_isar c a m bound =
+  let bound = Variables.push bound in
+    Printf.fprintf c "(! "; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c "::"; print_stp_isar c a (*Hashtbl.create 0(*FIXME*)*) false; Printf.fprintf c ". ";
+    match m with
+      | Ap(Forall(a'),Lam(_,m')) -> print_all_isar c a' m' bound; Printf.fprintf c ")"
+      | _ -> trm_to_isar c m bound; Printf.fprintf c ")"
+(* Prints an exist-term together with the corresponding lambda-term as an exists in Isar. *)
+and print_ex_isar c a m bound =
+ 	let bound = Variables.push bound in
+	  Printf.fprintf c "(? "; Printf.fprintf c "%s" (Variables.top bound);
+	  Printf.fprintf c "::"; print_stp_isar c a (*Hashtbl.create 0(*FIXME*)*) false;
+    Printf.fprintf c ". ";
+	  trm_to_isar c m bound; Printf.fprintf c ")"
+
+(*List the constants that occur in a term, as name-type pairs.
+  NOTE the resulting list doesn't contain duplicate constants.*)
+let rec consts_of_trm acc : trm -> ctx = function
+    Name (name, stp) ->
+    begin
+    try
+      let stp' = List.assoc name acc
+      in
+        if stp' <> stp then failwith "Different types for same constant?"
+        else acc
+    with Not_found ->
+      (name, stp) :: acc
+    end
+  | Lam (_, trm) -> consts_of_trm acc trm
+  | Ap (trm1, trm2) ->
+    let acc' = consts_of_trm acc trm1
+    in consts_of_trm acc' trm2
+  | False | Imp | Forall _ | Eq _ | Choice _
+  | True | Neg | Or | And | Iff | Exists _
+  | DB (_, _) -> acc
+
+(*Produce a list of names (strings) of the base types that occur in a type.
+  NOTE the resulting list doesn't contain duplicate names.*)
+let rec base_types acc : stp -> string list = function
+   Base name ->
+   if List.mem name acc then acc
+   else name :: acc
+ | Prop -> acc
+ | Ar (stp1, stp2) ->
+   let acc' = base_types acc stp1
+   in base_types acc' stp2
+
+(*Lists the base types referenced in a term.
+  NOTE the resulting list doesn't contain duplicate elements.*)
+let rec base_types_of_trm acc : trm -> string list = function
+    Name (name, stp) -> base_types acc stp
+  | Lam (stp, trm) ->
+    let acc' = base_types acc stp
+    in base_types_of_trm acc' trm
+  | Ap (trm1, trm2) ->
+    let acc' = base_types_of_trm acc trm1
+    in base_types_of_trm acc' trm2
+  | Forall stp | Eq stp | Choice stp | Exists stp | DB (_, stp) ->
+    base_types acc stp
+  | False | Imp | True | Neg | Or | And | Iff -> acc
